import sqlalchemy as sa

from aopi.models import AopiUser
from aopi.models.meta import Base


class AuthTokens(Base):
    user_id = sa.Column(sa.Integer(), sa.ForeignKey(AopiUser.id), nullable=False)
    refresh_token = sa.Column(sa.Text(), index=True, nullable=False)
    access_token = sa.Column(sa.Text(), nullable=False)

    @classmethod
    def add(cls, access_token: str, refresh_token: str, user_id: int) -> sa.sql.Insert:
        return cls.insert_query(
            access_token=access_token, refresh_token=refresh_token, user_id=user_id
        )



from fastapi import APIRouter, Depends, HTTPException
from starlette import status

from aopi.models import AopiUser, database
from aopi.models.auth_tokens import AuthTokens
from aopi.models.users import AopiUserModel
from aopi.routes.api.auth.dependencies import get_current_user_id, get_sub_from_jwt
from aopi.routes.api.auth.logic import (
    authenticate_user,
    create_access_token,
    create_refresh_token,
)
from aopi.routes.api.auth.schema import LoginRequestModel, LoginResponse, RefreshRequest

auth_router = APIRouter()


@auth_router.post("/login", response_model=LoginResponse)
async def get_new_token_pair(user_info: LoginRequestModel) -> LoginResponse:
    unauthorized = HTTPException(
        status_code=status.HTTP_401_UNAUTHORIZED,
        detail="Incorrect username or password",
        headers={"WWW-Authenticate": "Bearer"},
    )
    if user_info.username is None or user_info.password is None:
        raise unauthorized
    user = await authenticate_user(user_info.username, user_info.password)
    if user is None:
        raise unauthorized
    access_token = create_access_token(user_id=user.id)
    refresh_token = create_refresh_token(user_id=user.id)
    delete_query = AuthTokens.delete_query().where(AuthTokens.user_id == user.id)
    await database.execute(delete_query)
    add_query = AuthTokens.add(
        access_token=access_token, refresh_token=refresh_token, user_id=user.id
    )
    await database.execute(add_query)
    return LoginResponse(access_token=access_token, refresh_token=refresh_token)


@auth_router.post("/refresh", response_model=LoginResponse)
async def refresh_token_pair(refresh_data: RefreshRequest) -> LoginResponse:
    token_query = AuthTokens.select_query().where(
        AuthTokens.refresh_token == refresh_data.refresh_token
    )
    token = await database.fetch_one(token_query)
    incorrect_token = HTTPException(
        status_code=status.HTTP_401_UNAUTHORIZED,
        detail="Incorrect refresh token",
    )
    if token is None:
        raise incorrect_token
    user_id = get_sub_from_jwt(refresh_data.refresh_token, incorrect_token)

    access_token = create_access_token(user_id=user_id)
    refresh_token = create_refresh_token(user_id=user_id)
    delete_query = AuthTokens.delete_query().where(AuthTokens.user_id == user_id)
    await database.execute(delete_query)
    add_query = AuthTokens.add(
        access_token=access_token, refresh_token=refresh_token, user_id=user_id
    )
    await database.execute(add_query)
    return LoginResponse(access_token=access_token, refresh_token=refresh_token)


@auth_router.get(
    "/user", response_model=AopiUserModel, response_model_exclude={"password"}
)
async def get_current_user_info(
    user_id: int = Depends(get_current_user_id),
) -> AopiUserModel:
    user_not_found = HTTPException(
        status_code=status.HTTP_404_NOT_FOUND, detail="User not found"
    )
    if user_id is None:
        raise user_not_found
    find_query = AopiUser.select_query().where(AopiUser.id == user_id)
    if user_dict := await database.fetch_one(find_query):
        return AopiUserModel.from_orm(user_dict)
    raise user_not_found


@auth_router.post("/logout")
async def logout(user_id: int = Depends(get_current_user_id)) -> None:
    delete_query = AuthTokens.delete_query().where(AuthTokens.user_id == user_id)
    await database.execute(delete_query)



from typing import Optional

from aopi_index_builder import AopiContextBase, PluginManager

from aopi.models import AopiUser, AopiUserRole, database, metadata
from aopi.models.dict_proxy import DictProxy
from aopi.settings import settings
from aopi.utils.passwords import verify_password


async def get_user_id(username: str, password: str) -> Optional[int]:
    query = AopiUser.find(username, AopiUser.id, AopiUser.password)
    user = DictProxy(await database.fetch_one(query))
    if user.is_none():
        return None
    if not await verify_password(user.password, password):
        return None
    return user.id


async def check_user_permission(plugin: str, user_id: int, role: str) -> bool:
    test_query = AopiUserRole.has_role(user=user_id, plugin_name=plugin, role=role)
    perms = await database.fetch_val(test_query)
    return perms


plugin_manager = PluginManager(
    context=AopiContextBase(
        database=database,
        metadata=metadata,
        main_dir=settings.packages_dir,
        enable_users=settings.enable_users,
        get_user_id_function=get_user_id,
        check_user_permission=check_user_permission,
    )
)



import re
from typing import Any, Tuple, Type, Union

import databases
import sqlalchemy
import sqlalchemy as sa
from sqlalchemy.ext.declarative import as_declarative, declared_attr
from sqlalchemy.orm.attributes import InstrumentedAttribute

from aopi.settings import settings

database = databases.Database(settings.aopi_db_url)
metadata = sqlalchemy.MetaData()


@as_declarative(metadata=metadata)
class Base(object):
    __name__: str
    __table__: sa.Table
    __table_args__: Tuple[Any, ...]

    @declared_attr
    def __tablename__(self) -> str:
        """
        Converts CamelCase class name to snake_case table name.

        :return table name in snake_case.
        """
        name = re.sub(r"(.)([A-Z][a-z]+)", r"\1_\2", self.__name__)
        return re.sub("([a-z0-9])([A-Z])", r"\1_\2", name).lower()

    @declared_attr
    def id(self) -> Any:
        return sa.Column(
            sa.Integer,
            primary_key=True,
        )

    @classmethod
    def select_query(
        cls,
        *columns: Union[InstrumentedAttribute, Type["Base"]],
        use_labels: bool = False,
    ) -> sa.sql.Select:
        return sa.select(columns or [cls], use_labels=use_labels)

    @classmethod
    def insert_query(cls, **values: Any) -> sa.sql.Insert:
        return cls.__table__.insert().values(**values)

    @classmethod
    def update_query(cls) -> sa.sql.Update:
        return cls.__table__.update()

    @classmethod
    def delete_query(cls) -> sa.sql.Delete:
        return cls.__table__.delete()



try:
    from aopi.runners.gunicorn_runner import run_app
except ImportError:
    from aopi.runners.uvicorn_runner import run_app


def main() -> None:
    run_app()


if __name__ == "__main__":
    main()



from fastapi import FastAPI
from loguru import logger
from starlette.responses import UJSONResponse
from starlette.staticfiles import StaticFiles

from aopi import FRONTEND_DIR
from aopi.application.lifetime import shutdown, startup
from aopi.application.plugin_manager import plugin_manager
from aopi.models import create_db
from aopi.settings import settings
from aopi.utils.logging import configure_logging


def get_application() -> FastAPI:
    from aopi.routes import api_router

    configure_logging()
    app = FastAPI(
        title="Another One Package Index",
        default_response_class=UJSONResponse,
    )
    app.on_event("startup")(startup)
    app.on_event("shutdown")(shutdown)
    plugin_manager.load()
    plugin_manager.add_routes(app)
    create_db()
    logger.debug("DB initialized")
    app.include_router(api_router, prefix="/api")
    if not settings.no_ui:
        app.mount("/", StaticFiles(directory=FRONTEND_DIR, html=True), name="static")
    logger.debug("Routes mounted")
    logger.info("Worker is up")
    return app



import asyncio
from typing import Any, Callable

from argon2 import PasswordHasher
from argon2.exceptions import VerifyMismatchError

argon_hasher = PasswordHasher()


async def run_async(function: Callable[..., Any], *args: Any) -> Any:
    loop = asyncio.get_event_loop()
    return await loop.run_in_executor(None, function, *args)


async def hash_password(password: str) -> str:
    return await run_async(argon_hasher.hash, password)


async def verify_password(hashed_password: str, password: str) -> bool:
    try:
        return await run_async(argon_hasher.verify, hashed_password, password)
    except VerifyMismatchError:
        return False



import logging
from typing import Any, Dict, Optional

from gunicorn.app.base import BaseApplication
from gunicorn.config import Config
from gunicorn.glogging import Logger

from aopi.settings import settings


class StubbedGunicornLogger(Logger):
    def setup(self, cfg: Config) -> None:
        handler = logging.NullHandler()
        self.error_logger = logging.getLogger("gunicorn.error")
        self.error_logger.addHandler(handler)
        self.access_logger = logging.getLogger("gunicorn.access")
        self.access_logger.addHandler(handler)
        self.error_logger.setLevel(settings.log_level.value)
        self.access_logger.setLevel(settings.log_level.value)


class StandaloneApplication(BaseApplication):
    def init(self, parser: Any, opts: Any, args: Any) -> None:
        super(StandaloneApplication, self).init(parser, opts, args)

    def __init__(self, run_options: Optional[Dict[str, Any]] = None):
        self.options = run_options or {}
        super().__init__()

    def load_config(self) -> None:
        config = {
            key: value
            for key, value in self.options.items()
            if key in self.cfg.settings and value is not None
        }
        for key, value in config.items():
            self.cfg.set(key.lower(), value)

    def load(self) -> Any:
        import aopi.application

        return aopi.application.get_application()


def run_app() -> None:
    settings.pprint()
    options = {
        "bind": f"{settings.host}:{settings.port}",
        "workers": settings.workers_count,
        "worker_class": "uvicorn.workers.UvicornWorker",
        "pidfile": str(settings.pid_file),
        "reload": settings.reload,
        "accesslog": "-",
        "errorlog": "-",
        "logger_class": StubbedGunicornLogger,
        "timeout": 0,
    }
    StandaloneApplication(options).run()



from datetime import datetime, timedelta
from typing import Any, Dict, Optional

from jose import jwt

from aopi.models import AopiUser, database
from aopi.models.users import AopiUserModel
from aopi.settings import settings
from aopi.utils.passwords import verify_password


def create_jwt(data: Dict[str, Any], expires_delta: Optional[timedelta] = None) -> str:
    to_encode = data.copy()
    if expires_delta:
        expire = datetime.utcnow() + expires_delta
    else:
        expire = datetime.utcnow() + timedelta(minutes=15)
    to_encode.update({"exp": expire})
    encoded_jwt = jwt.encode(
        to_encode, settings.jwt_secret, algorithm=settings.jwt_algorithm
    )
    return encoded_jwt


def create_access_token(user_id: int) -> str:
    access_token_expires = timedelta(minutes=30)
    return create_jwt(data={"sub": str(user_id)}, expires_delta=access_token_expires)


def create_refresh_token(user_id: int) -> str:
    refresh_token_expires = timedelta(days=30)
    return create_jwt(data={"sub": str(user_id)}, expires_delta=refresh_token_expires)


async def authenticate_user(username: str, password: str) -> Optional[AopiUserModel]:
    user_query = AopiUser.find(username=username)
    found_users = await database.fetch_all(user_query)
    if len(found_users) != 1:
        return None
    user = AopiUserModel.from_orm(found_users[0])
    if not await verify_password(user.password, password):
        return None
    return user



import os
import secrets
from pathlib import Path
from tempfile import gettempdir
from typing import Union

from databases import DatabaseURL
from loguru import logger
from pydantic import BaseModel, Field
from tabulate import tabulate

from aopi.arg_parser import LogLevel, parse_args

TEMP_DIR = Path(gettempdir())


class Settings(BaseModel):
    packages_dir: Path = Field(TEMP_DIR / "aopi/packages")
    aopi_db_url: str = Field(f"sqlite:///{TEMP_DIR}/aopi/db.sqlite")
    pid_file: Path = Field(TEMP_DIR / "aopi/server.pid")
    secret_file: Path = Field(TEMP_DIR / "aopi/secret")
    jwt_secret: str = ""
    jwt_algorithm: str = "HS256"
    log_level: LogLevel = Field(LogLevel.info)
    workers_count: int = 4
    reload: bool = False
    port: int = 8000
    host: str = "0.0.0.0"
    no_ui: bool = False
    enable_users: bool = False

    def prepare(self) -> None:
        def create_dir(filename: Union[Path, str], parent: bool = True) -> None:
            base_dir = filename
            if parent:
                base_dir = os.path.dirname(filename)
            if not os.path.exists(base_dir):
                logger.debug(f"Creating directory {base_dir}")
                os.makedirs(base_dir)

        create_dir(self.pid_file)
        create_dir(self.secret_file)
        create_dir(self.packages_dir, False)
        db_url = DatabaseURL(self.aopi_db_url)
        if db_url.dialect == "sqlite" and db_url.hostname is None:
            create_dir(db_url.database, True)
        if not self.secret_file.exists():
            self.secret_file.write_text(secrets.token_urlsafe(128))
        self.jwt_secret = self.secret_file.read_text()

    @classmethod
    def from_args(cls) -> "Settings":
        cli_args = parse_args()
        params = {k: v for k, v in cli_args.__dict__.items() if v is not None}
        return Settings(**params)

    def pprint(self) -> None:
        print("Current settings:")
        safe_data = self.dict(exclude={"jwt_secret"})
        print(tabulate(safe_data.items(), stralign="left", tablefmt="plain"))
        print()


settings = Settings.from_args()
settings.prepare()



from typing import Any, Dict

from fastapi import APIRouter

from aopi.routes.api import auth_router, packages_router
from aopi.settings import settings

api_router = APIRouter()
api_router.include_router(packages_router, tags=["packages"])
if settings.enable_users:
    api_router.include_router(auth_router, prefix="/auth", tags=["auth"])


@api_router.get("/system", tags=["system"])
def get_backend_info() -> Dict[str, Any]:
    """
    # Get current backend configuration.

    This route is used by aopi-frontend application.
    """
    return {"users_enabled": settings.enable_users}



import sqlalchemy

from aopi.models.meta import database, metadata
from aopi.models.roles import AopiRole
from aopi.models.user_roles import AopiUserRole
from aopi.models.users import AopiUser

__all__ = ["create_db", "database", "AopiUser", "AopiRole", "AopiUserRole"]


def create_db() -> None:
    engine = sqlalchemy.create_engine(str(database.url))
    metadata.create_all(engine)



from typing import Optional

from fastapi import Depends, HTTPException
from fastapi.security import OAuth2PasswordBearer
from jose import JWTError, jwt
from starlette import status

from aopi.settings import settings

oauth2_scheme = OAuth2PasswordBearer(tokenUrl="/api/auth/login")


def get_sub_from_jwt(token: str, error: Exception) -> int:
    try:
        payload = jwt.decode(
            token, settings.jwt_secret, algorithms=[settings.jwt_algorithm]
        )
        user_id: int = payload.get("sub")
        if user_id is None:
            raise error
    except JWTError:
        raise error
    return user_id


if settings.enable_users:

    def get_current_user_id(token: str = Depends(oauth2_scheme)) -> Optional[int]:
        credentials_exception = HTTPException(
            status_code=status.HTTP_401_UNAUTHORIZED,
            detail="Could not validate credentials",
            headers={"WWW-Authenticate": "Bearer"},
        )
        return int(get_sub_from_jwt(token, credentials_exception))


else:

    def get_current_user_id() -> Optional[int]:  # type: ignore
        return None



import sqlalchemy as sa
from sqlalchemy import UniqueConstraint

from aopi.models.meta import Base


class AopiRole(Base):
    role = sa.Column(sa.String, nullable=False)
    plugin_name = sa.Column(sa.String, nullable=False)

    __table_args__ = (UniqueConstraint(role, plugin_name),)



from typing import Any, Dict, Optional


class DictProxy:
    def __init__(self, values: Optional[Dict[str, Any]]) -> None:
        self.__init_dict = values
        self.__obj_dict = dict()
        if values is None:
            return
        obj_dict = dict()
        for key, value in values.items():
            if isinstance(value, Dict):
                value = DictProxy(value)
            obj_dict[key] = value
        self.__obj_dict = obj_dict

    def is_none(self) -> bool:
        return self.__init_dict is None

    def __getattr__(self, item: str) -> Any:
        return self.__obj_dict.get(item)

    def __repr__(self) -> str:
        body = None
        if self.__init_dict:
            body = ",".join([f"{key}:{val}" for key, val in self.__init_dict.items()])
        return f"<Dict proxy {body}>"

    def __str__(self) -> str:
        return self.__repr__()



from typing import Union

import sqlalchemy as sa
from sqlalchemy import and_, exists
from sqlalchemy.orm.attributes import InstrumentedAttribute

from aopi.models.meta import Base
from aopi.models.roles import AopiRole
from aopi.models.users import AopiUser


class AopiUserRole(Base):
    user_id = sa.Column(sa.Integer, sa.ForeignKey(AopiUser.id))
    role_id = sa.Column(sa.Integer, sa.ForeignKey(AopiRole.id))

    __table_args__ = (sa.UniqueConstraint(user_id, role_id),)

    @classmethod
    def get_user_roles(
        cls, user: Union[str, int], *fields: InstrumentedAttribute
    ) -> sa.sql.Select:
        if isinstance(user, int):
            user_id = user
        else:
            user_id = AopiUser.find(user, AopiUser.id)
        return AopiRole.select_query(*fields).where(
            AopiRole.id.in_(
                cls.select_query(AopiUserRole.role_id).where(
                    AopiUserRole.user_id == user_id
                )
            )
        )

    @classmethod
    def has_role(
        cls, *, user: Union[str, int], plugin_name: str, role: str
    ) -> sa.sql.Select:
        if isinstance(user, int):
            user_id = user
        else:
            user_id = AopiUser.find(user, AopiUser.id)
        role_id = AopiRole.select_query(AopiRole.id).where(
            and_(AopiRole.role == role, AopiRole.plugin_name == plugin_name)
        )
        user_role = cls.select_query(AopiUserRole.id).where(
            and_(AopiUserRole.user_id == user_id, AopiUserRole.role_id == role_id)
        )
        return sa.select([exists(user_role)])



import logging
import sys
from typing import Optional, Union

from loguru import logger

from aopi.arg_parser import LogLevel
from aopi.settings import settings


class InterceptHandler(logging.Handler):
    def emit(self, record: logging.LogRecord) -> None:
        # Get corresponding Loguru level if it exists
        level: Optional[Union[str, int]] = None
        try:
            level = logger.level(record.levelname).name
        except ValueError:
            level = record.levelno

        # Find caller from where originated the logged message
        frame, depth = logging.currentframe(), 2
        while frame.f_code.co_filename == logging.__file__:
            frame = frame.f_back  # type: ignore
            depth += 1

        logger.opt(depth=depth, exception=record.exc_info).log(
            level, record.getMessage()
        )


def configure_logging() -> None:
    intercept_handler = InterceptHandler()
    logger.remove()
    logger.add(sys.stderr, level=settings.log_level.value)
    logging.root.setLevel(LogLevel.error.value)

    seen = {"sqlalchemy", "databases", "aiosqlite"}
    for name in [
        *logging.root.manager.loggerDict.keys(),  # type: ignore
        "aopi",
        "gunicorn",
        "gunicorn.access",
        "gunicorn.error",
        "fastapi",
        "uvicorn",
        "uvicorn.server",
        "uvicorn.access",
        "uvicorn.error",
    ]:
        if name not in seen:
            seen.add(name)
            tmp_logger = logging.getLogger(name)
            tmp_logger.setLevel(settings.log_level.value)
            tmp_logger.handlers.clear()
            tmp_logger.propagate = False
            tmp_logger.handlers = [intercept_handler]



from collections import defaultdict

from loguru import logger

from aopi.application.plugin_manager import plugin_manager
from aopi.models import AopiRole, AopiUser, AopiUserRole
from aopi.models.dict_proxy import DictProxy
from aopi.models.meta import database
from aopi.settings import settings
from aopi.utils.passwords import hash_password


async def create_admin() -> None:
    """
    Create admin user if it's not exits.

    """
    hashed_pass = await hash_password("admin")
    find_query = AopiUser.find("admin")
    insert_query = AopiUser.create("admin", hashed_pass)
    if await database.fetch_one(find_query):
        logger.debug("Admin already exists")
        return
    logger.debug("Creating admin")
    await database.execute(insert_query)


async def create_missing_roles() -> None:
    """
    Find missing roles in database and add them.

    """
    existing_roles = map(DictProxy, await database.fetch_all(AopiRole.select_query()))
    existing_mapping = defaultdict(list)
    for role in existing_roles:
        existing_mapping[role.plugin_name].append(role.role)
    missing_roles = list()
    for plugin in plugin_manager.plugins_map.values():
        existing_plugin_roles = set(existing_mapping.get(plugin.package_name) or [])
        for role in set(plugin.roles) - existing_plugin_roles:
            missing_roles.append({"plugin_name": plugin.package_name, "role": role})
    logger.debug(f"Found {len(missing_roles)} missing roles.")
    await database.execute_many(AopiRole.insert_query(), missing_roles)


async def add_roles_to_admin() -> None:
    """
    Add all roles from database to admin.

    """
    find_query = AopiUser.find("admin", AopiUser.id)
    user_id = await database.fetch_val(find_query)
    admin_roles = AopiUserRole.get_user_roles(user_id, AopiRole.id)
    missing_roles = AopiRole.select_query(AopiRole.id).where(
        ~AopiRole.id.in_(admin_roles)
    )
    roles = [
        {"role_id": role.id, "user_id": user_id}
        for role in map(DictProxy, await database.fetch_all(missing_roles))
    ]
    logger.debug(f"Found {len(roles)} missing roles for admin user")
    await database.execute_many(AopiUserRole.insert_query(), roles)


async def fill_db() -> None:
    """
    Create necessary items on startup.
    Like admin user and missing roles from plugins.

    """
    if settings.enable_users:
        await create_admin()
        await create_missing_roles()
        await add_roles_to_admin()


async def startup() -> None:
    await database.connect()
    logger.info("Database connected")
    await fill_db()


async def shutdown() -> None:
    await database.disconnect()



import sqlalchemy as sa
from pydantic.main import BaseConfig, BaseModel
from sqlalchemy.orm.attributes import InstrumentedAttribute

from aopi.models.meta import Base


class AopiUser(Base):
    username = sa.Column(sa.String(length=255), unique=True, nullable=False)
    password = sa.Column(sa.Text(), nullable=False)

    @classmethod
    def create(cls, username: str, password: str) -> sa.sql.Insert:
        return cls.insert_query(username=username, password=password)

    @classmethod
    def find(cls, username: str, *fields: InstrumentedAttribute) -> sa.sql.Select:
        return cls.select_query(*fields).where(AopiUser.username == username)


class AopiUserModel(BaseModel):
    id: int
    username: str
    password: str

    class Config(BaseConfig):
        orm_mode = True



import argparse
from enum import Enum
from pathlib import Path

from argparse_utils import enum_action


class LogLevel(str, Enum):
    info = "INFO"
    debug = "DEBUG"
    error = "ERROR"
    notset = "NOTSET"
    warning = "WARNING"
    critical = "CRITICAL"


def parse_args() -> argparse.Namespace:
    parser = argparse.ArgumentParser()
    parser.add_argument("--host", type=str, dest="host", help="Application host")
    parser.add_argument("--port", type=int, dest="port", help="Application port")
    parser.add_argument(
        "-r",
        "--reload",
        action="store_true",
        dest="reload",
        help="Reload application on change (Development only)",
    )
    parser.add_argument(
        "-w",
        "--workers",
        type=int,
        dest="workers_count",
        help="Number of application processes. (Only for Gunicorn workers)",
    )
    parser.add_argument(
        "--pid-file",
        type=str,
        dest="pid_file",
        help="Path to file where to store "
        "server application ID. (Only for Gunicorn workers)",
    )
    parser.add_argument(
        "-d", "--database", type=str, dest="aopi_db_url", help="Database file"
    )
    parser.add_argument(
        "-p",
        "--packages-dir",
        type=str,
        dest="packages_dir",
        help="Path to folder where to store actual simple",
    )
    parser.add_argument(
        "--log-level",
        action=enum_action(LogLevel, str.lower),
        dest="log_level",
        help="Logging level for application",
    )
    parser.add_argument(
        "-n",
        "--no-ui",
        action="store_true",
        dest="no_ui",
        help="Turn off aopi web-interface",
    )
    parser.add_argument(
        "-u" "--users",
        action="store_true",
        dest="enable_users",
        help="Enable user system",
    )
    parser.add_argument(
        "--secret-file",
        type=Path,
        dest="secret_file",
        help="Path to file where secret stored",
    )
    return parser.parse_args()



from typing import Optional

from pydantic import BaseModel


class RefreshRequest(BaseModel):
    refresh_token: str


class LoginRequestModel(BaseModel):
    username: Optional[str]
    password: Optional[str]


class LoginResponse(BaseModel):
    access_token: str
    refresh_token: str
    token_type: str = "bearer"



import uvicorn

from aopi.settings import settings


def run_app() -> None:
    settings.pprint()
    uvicorn.run(
        app="aopi.application:get_application",
        factory=True,
        host=settings.host,
        port=settings.port,
        reload=settings.reload,
        workers=settings.workers_count,
        log_config=None,
    )



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['aopi',
 'aopi.application',
 'aopi.models',
 'aopi.routes',
 'aopi.routes.api',
 'aopi.routes.api.auth',
 'aopi.routes.api.packages',
 'aopi.runners',
 'aopi.utils']

package_data = \
{'': ['*']}

install_requires = \
['Jinja2>=2.11.2,<3.0.0',
 'aiofiles>=0.6.0,<0.7.0',
 'aiosqlite>=0.16.0,<0.17.0',
 'aopi-index-builder>=0.1.40,<0.2.0',
 'argon2-cffi>=20.1.0,<21.0.0',
 'argparse-utils>=1.3.0,<2.0.0',
 'fastapi>=0.63.0,<0.64.0',
 'httptools>=0.1.1,<0.2.0',
 'loguru>=0.5.3,<0.6.0',
 'pydantic>=1.7.3,<2.0.0',
 'python-jose>=3.2.0,<4.0.0',
 'python-multipart>=0.0.5,<0.0.6',
 'tabulate>=0.8.7,<0.9.0',
 'ujson>=4.0.1,<5.0.0',
 'uvicorn>=0.13.3,<0.14.0']

extras_require = \
{'mysql': ['aiomysql>=0.0.21,<0.0.22', 'mysqlclient>=2.0.2,<3.0.0'],
 'postgre': ['asyncpg>=0.21.0,<0.22.0', 'psycopg2>=2.8.6,<3.0.0'],
 'python': ['aopi-python>=0.1.21,<0.2.0'],
 'unix': ['uvloop>=0.14.0,<0.15.0', 'gunicorn>=20.0.4,<21.0.0']}

entry_points = \
{'console_scripts': ['aopi = aopi.main:main']}

setup_kwargs = {
    'name': 'aopi',
    'version': '0.1.8',
    'description': '',
    'long_description': None,
    'author': 'Pavel Kirilin',
    'author_email': 'win10@list.ru',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'extras_require': extras_require,
    'entry_points': entry_points,
    'python_requires': '>=3.9,<4.0',
}


setup(**setup_kwargs)



from typing import Any, List, Optional

from aopi_index_builder import (
    PackageVersion,
    PluginFullPackageInfo,
    PluginPackagePreview,
)
from fastapi import APIRouter, Depends, HTTPException

from aopi.application.plugin_manager import plugin_manager
from aopi.routes.api.auth.dependencies import get_current_user_id

packages_router = APIRouter()


@packages_router.get("/languages", response_model=List[str])
async def get_available_languages() -> List[str]:
    return plugin_manager.get_languages()


@packages_router.get("/packages", response_model=List[PluginPackagePreview])
async def find_package(
    page: int = 0,
    name: str = "",
    limit: int = 100,
    language: Optional[str] = None,
    user_id: Optional[int] = Depends(get_current_user_id),
) -> List[PluginPackagePreview]:
    return await plugin_manager.find_package(
        page=page, limit=limit, user_id=user_id, package_name=name, language=language
    )


@packages_router.get("/package", response_model=PluginFullPackageInfo)
async def get_package_info(
    plugin_name: str,
    package_id: Any,
    user_id: Optional[int] = Depends(get_current_user_id),
) -> PluginFullPackageInfo:
    info = await plugin_manager.get_package_info(
        user_id=user_id, plugin_name=plugin_name, package_id=package_id
    )
    if info is None:
        raise HTTPException(status_code=404, detail="Package not found")
    return info


@packages_router.get("/versions", response_model=List[PackageVersion])
async def get_package_versions(
    plugin_name: str,
    package_id: Any,
    user_id: Optional[int] = Depends(get_current_user_id),
) -> List[PackageVersion]:
    return await plugin_manager.get_package_versions(
        user_id=user_id, plugin_name=plugin_name, package_id=package_id
    )
